@@ -28,6 +28,7 @@ using System;
 using System.Collections;
 using System.IO;
 using System.Reflection;
+using System.Runtime.InteropServices;
 using System.Threading;
 
 using Gtk;
@@ -110,8 +111,12 @@ namespace Beagle.Daemon {
 				"  --debug-memory\tWrite out debugging information about memory use.\n" +
 				"  --indexing-test-mode\tRun in foreground, and exit when fully indexed.\n" +
 				"  --indexing-delay\tTime to wait before indexing.  (Default 60 seconds)\n" +
-				"  --deny-backend\tDeny a specific backend.\n" +
-				"  --allow-backend\tAllow a specific backend.\n" +
+				"  --backend\t\tConfigure which backends to use.  Specifically:\n" +
+				"    --backend <name>\tOnly start backend 'name'\n" +
+				"    --backend +<name>\tAdditionally start backend 'name'\n" +
+				"    --backend -<name>\tDisable backend 'name'\n" +
+				"  --allow-backend\t(DEPRECATED) Start only the specific backend.\n" +
+				"  --deny-backend\t(DEPRECATED) Deny a specific backend.\n" +
 				"  --list-backends\tList all the available backends.\n" +
 				"  --add-static-backend\tAdd a static backend by path.\n" + 
 				"  --disable-scheduler\tDisable the use of the scheduler.\n" +
@@ -281,13 +286,42 @@ namespace Beagle.Daemon {
 					arg_fg = true;
 					break;
 
+				case "--backend":
+					if (next_arg == null) {
+						Console.WriteLine ("--backend requires a backend name");
+						Environment.Exit (1);
+						break;
+					}
+
+					if (next_arg.StartsWith ("--")) {
+						Console.WriteLine ("--backend requires a backend name. Invalid name '{0}'", next_arg);
+						Environment.Exit (1);
+						break;
+					}
+
+					if (next_arg [0] != '+' && next_arg [0] != '-')
+						QueryDriver.OnlyAllow (next_arg);
+					else {
+						if (next_arg [0] == '+')
+							QueryDriver.Allow (next_arg.Substring (1));
+						else
+							QueryDriver.Deny (next_arg.Substring (1));
+					}
+
+					++i; // we used next_arg
+					break;
+				
 				case "--allow-backend":
+					// --allow-backend is deprecated, use --backends 'name' instead
+					// it will disable reading the list of enabled/disabled backends
+					// from conf and start the backend given
 					if (next_arg != null)
-						QueryDriver.Allow (next_arg);
+						QueryDriver.OnlyAllow (next_arg);
 					++i; // we used next_arg
 					break;
 					
 				case "--deny-backend":
+					// deprecated: use --backends -'name' instead
 					if (next_arg != null)
 						QueryDriver.Deny (next_arg);
 					++i; // we used next_arg
@@ -358,6 +392,14 @@ namespace Beagle.Daemon {
 				
 
 			// Bail out if we are trying to run as root
+			if (Environment.UserName == "root" && Environment.GetEnvironmentVariable ("SUDO_USER") != null) {
+				Console.WriteLine ("You appear to be running beagle using sudo.  This can cause problems with");
+				Console.WriteLine ("permissions in your .beagle and .wapi directories if you later try to run");
+				Console.WriteLine ("as an unprivileged user.  If you need to run beagle as root, please use");
+				Console.WriteLine ("'su -c' instead.");
+				Environment.Exit (-1);
+			}
+
 			if (Environment.UserName == "root" && ! Conf.Daemon.AllowRoot) {
 				Console.WriteLine ("You can not run beagle as root.  Beagle is designed to run from your own");
 				Console.WriteLine ("user account.  If you want to create multiuser or system-wide indexes, use");
@@ -367,6 +409,13 @@ namespace Beagle.Daemon {
 				Environment.Exit (-1);
 			}
 
+			try {
+				string tmp = PathFinder.HomeDir;
+			} catch (Exception e) {
+				Console.WriteLine ("Unable to start the daemon: {0}", e.Message);
+				Environment.Exit (-1);
+			}
+
 			MainLoopThread = Thread.CurrentThread;
 
 			Log.Initialize (PathFinder.LogDir,
@@ -378,6 +427,8 @@ namespace Beagle.Daemon {
 					arg_fg);
 
 			Logger.Log.Info ("Starting Beagle Daemon (version {0})", ExternalStringsHack.Version);
+
+			Logger.Log.Info ("Running on {0}", SystemInformation.MonoRuntimeVersion);
 			
 			Logger.Log.Debug ("Command Line: {0}",
 					  Environment.CommandLine != null ? Environment.CommandLine : "(null)");
@@ -394,7 +445,12 @@ namespace Beagle.Daemon {
 			// Do BEAGLE_EXERCISE_THE_DOG_HARDER-related processing.
 			ExerciseTheDogHarder ();
 
-			Application.InitCheck ("beagled", ref args);
+			if (Application.InitCheck ("beagled", ref args))
+				Logger.Log.Debug ("Established a connection to the X server");
+			else
+				Logger.Log.Debug ("Unable to establish a connection to the X server");
+
+			XSetIOErrorHandler (BeagleXIOErrorHandler);
 
 			// Defer all actual startup until the main loop is
 			// running.  That way shutdowns during the startup
@@ -421,7 +477,28 @@ namespace Beagle.Daemon {
 				ExceptionHandlingThread.SpewLiveThreads ();
 			}
 		}
-		
+
+		/////////////////////////////////////////////////////////////////////////////
+
+		private delegate int XIOErrorHandler (IntPtr display);
+
+		[DllImport ("libX11.so.6")]
+		extern static private int XSetIOErrorHandler (XIOErrorHandler handler);
+
+		private static int BeagleXIOErrorHandler (IntPtr display)
+		{
+			Logger.Log.Debug ("Lost our connection to the X server!  Trying to shut down gracefully");
+
+			if (! Shutdown.ShutdownRequested)
+				Shutdown.BeginShutdown ();
+
+			Logger.Log.Debug ("Xlib is forcing us to exit!");
+
+			ExceptionHandlingThread.SpewLiveThreads ();
+	
+			// Returning will cause xlib to exit immediately.
+			return 0;
+		}
 
 		/////////////////////////////////////////////////////////////////////////////
 
@@ -433,8 +510,9 @@ namespace Beagle.Daemon {
 			// Set up our signal handler
 			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGINT, OurSignalHandler);
 			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGTERM, OurSignalHandler);
-			if (Environment.GetEnvironmentVariable("BEAGLE_THERE_BE_NO_QUITTIN") == null)
-				Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGQUIT, OurSignalHandler);
+
+			// Ignore SIGPIPE
+			Mono.Unix.Native.Stdlib.signal (Mono.Unix.Native.Signum.SIGPIPE, Mono.Unix.Native.Stdlib.SIG_IGN);
 		}
 
 		// Our handler triggers an orderly shutdown when it receives a signal.
@@ -450,12 +528,7 @@ namespace Beagle.Daemon {
 			if (signal < 0)
 				return;
 
-			Logger.Log.Debug ("Handling signal {0}", signal);
-
-			if (signal == (int) Mono.Unix.Native.Signum.SIGQUIT) {
-				ExceptionHandlingThread.AbortThreads ();
-				return;
-			}
+			Logger.Log.Debug ("Handling signal {0} ({1})", signal, (Mono.Unix.Native.Signum) signal);
 
 			bool first_signal = false;
 			if (signal_time == DateTime.MinValue) {
