@@ -26,7 +26,10 @@
 //
 
 using System;
+using System.IO;
+using System.Text;
 using System.Reflection;
+using System.Collections;
 
 using Beagle;
 using Beagle.Daemon;
@@ -46,7 +49,9 @@ class InfoTool {
 			"  --daemon-version\t\tPrint the version of the running daemon.\n" +
 			"  --status\t\t\tDisplay status of the running daemon.\n" +
 			"  --index-info\t\t\tDisplay statistics of the Beagle indexes.\n" +
+			"  --list-backends\t\tList the currently available backends.\n" +
 			"  --list-filters\t\tList the currently available filters.\n" +
+			"  --list-static-indexes\t\tList the available static indexes.\n" +
 			"  --help\t\t\tPrint this usage message.\n";
 
 		Console.WriteLine (usage);
@@ -61,6 +66,10 @@ class InfoTool {
 
 		if (Array.IndexOf (args, "--list-filters") > -1)
 			PrintFilterInformation ();
+		else if (Array.IndexOf (args, "--list-backends") > -1)
+			PrintBackendInformation ();
+		else if (Array.IndexOf (args, "--list-static-indexes") > -1)
+			PrintStaticIndexInformation ();
 		else
 			return PrintDaemonInformation (args);
 
@@ -95,7 +104,7 @@ class InfoTool {
 
 		return 0;
 	}
-	
+
 	private static void PrintFilterInformation ()
 	{
 		ReflectionFu.ScanEnvironmentForAssemblies ("BEAGLE_FILTER_PATH", PathFinder.FilterDir, PrintFilterDetails);
@@ -103,6 +112,7 @@ class InfoTool {
 
 	static void PrintFilterDetails (Assembly assembly)
 	{
+		StringBuilder sb = new StringBuilder ();
 		foreach (Type t in ReflectionFu.ScanAssemblyForClass (assembly, typeof (Filter))) {
 			Filter filter = null;
 
@@ -123,18 +133,68 @@ class InfoTool {
 			else
 				name = t.FullName;
 
-			Console.WriteLine (name + " - Version " + filter.Version + " (" + assembly.Location + ")");
+			sb.Length = 0;
+			sb.Append (name + " - Version " + filter.Version + " (" + assembly.Location + ")\n");
+			bool has_filter = false;
 
 			foreach (FilterFlavor flavor in filter.SupportedFlavors) {
-				if (flavor.MimeType != null)
-					Console.WriteLine ("  - " + flavor.MimeType);
-				if (flavor.Extension != null)
-					Console.WriteLine ("  - *" + flavor.Extension);
+				if (flavor.MimeType != null && (! flavor.MimeType.StartsWith ("beagle"))) {
+					sb.Append ("  - " + flavor.MimeType + "\n");
+					has_filter = true;
+				}
+				if (flavor.Extension != null) {
+					sb.Append ("  - *" + flavor.Extension + "\n");
+					has_filter = true;
+				}
 			}
 
-			Console.WriteLine ();
+			if (has_filter)
+				Console.WriteLine (sb.ToString ());
 		}
 	}
+
+	private static void PrintBackendInformation ()
+	{
+		ArrayList assemblies = ReflectionFu.ScanEnvironmentForAssemblies ("BEAGLE_BACKEND_PATH", PathFinder.BackendDir);
+
+		// Add BeagleDaemonLib if it hasn't already been added.
+		bool found_daemon_lib = false;
+		foreach (Assembly assembly in assemblies) {
+			if (assembly.GetName ().Name == "BeagleDaemonLib") {
+				found_daemon_lib = true;
+				break;
+			}
+		}
+
+		if (!found_daemon_lib) {
+			try {
+				assemblies.Add (Assembly.LoadFrom (Path.Combine (PathFinder.PkgLibDir, "BeagleDaemonLib.dll")));
+			} catch (FileNotFoundException) {
+				Console.WriteLine ("WARNING: Could not find backend list.");
+				Environment.Exit (1);
+			}
+		}
+
+		foreach (Assembly assembly in assemblies) {
+			foreach (Type type in ReflectionFu.ScanAssemblyForInterface (assembly, typeof (Beagle.Daemon.IQueryable))) {
+				foreach (Beagle.Daemon.QueryableFlavor flavor in ReflectionFu.ScanTypeForAttribute (type, typeof (Beagle.Daemon.QueryableFlavor))) {
+					Console.WriteLine ("{0,-20} (" + assembly.Location + ")", flavor.Name);
+				}
+			}
+		}
+	}
+
+	private static void PrintStaticIndexInformation ()
+	{
+		try {
+			foreach (DirectoryInfo index_dir in new DirectoryInfo (PathFinder.SystemIndexesDir).GetDirectories ())
+				Console.WriteLine ("[System index] " + index_dir.Name + " (" + index_dir.FullName + ")");
+		} catch (DirectoryNotFoundException) { }
+
+		foreach (string index_path in Conf.Daemon.StaticQueryables)
+			Console.WriteLine ("[User index]   " + index_path);
+	}
+	
 }
 		
 		
